import logging

from housing.config import LOGS_PATH


def get_logger(logger_name):
    """
    get_logger : Generate logger object which will log the events in the module during the runtime

    Arguments:
        logger_name {str} -- Name of the process

    Returns:
        logger object -- Logger object to be used to log the events
    """
    logger = logging.getLogger(logger_name)
    logger.setLevel(logging.DEBUG)

    # open the text file used for logging in the append mode
    f_handler = logging.FileHandler(filename=LOGS_PATH, mode="w")
    # string format of the text to put in the log file
    f_format = logging.Formatter(
        "%(asctime)s - %(filename)s - %(funcName)s - %(lineno)d - %(levelname)s - %(message)s",
        datefmt="%Y-%m-%d %H:%M:%S",
    )
    # assign the string format and file handler to the logger object
    f_handler.setFormatter(f_format)
    logger.addHandler(f_handler)

    logger.addHandler(logging.StreamHandler())

    return logger



import argparse
import logging
import os
import tarfile

import pandas as pd
import requests
from sklearn.model_selection import train_test_split

from housing.config import *

parser = argparse.ArgumentParser()
parser.add_argument(
    "--path", help="Path to store the data files", default=DEFAULT_HOUSING_PATH
)
args = parser.parse_args()

HOUSING_PATH = args.path

logging.basicConfig(level=logging.WARN)
logger = logging.getLogger(__name__)


with requests.get(HOUSING_URL, stream=True) as rx, tarfile.open(
    fileobj=rx.raw, mode="r:gz"
) as tarobj:
    tarobj.extractall(HOUSING_PATH)


def load_housing_data(housing_path=HOUSING_PATH):
    csv_path = os.path.join(housing_path, "housing.csv")
    return pd.read_csv(csv_path)


housing = load_housing_data()

train_set, test_set = train_test_split(housing, test_size=0.2, random_state=42)

train_set.to_csv(os.path.join(HOUSING_PATH, "train.csv"))
test_set.to_csv(os.path.join(HOUSING_PATH, "test.csv"))



""" This fiule contains all functions to be used"""


import numpy as np
import pandas as pd
from sklearn.impute import SimpleImputer
from sklearn.metrics import mean_absolute_error, mean_squared_error, r2_score


def eval_metrics(actual, pred):
    """Evaluation Metric for calculating Accuracy metrics.

    Args:
        actual (int): Actual Data
        pred (int): Predicted Data

    Returns:
        tuple: (rmse, mae, r2)
    """
    rmse = np.sqrt(mean_squared_error(actual, pred))
    mae = mean_absolute_error(actual, pred)
    r2 = r2_score(actual, pred)
    return rmse, mae, r2


def preprocess(data):
    """_summary_

    Args:
        data (DataFrame): Input data to be processed

    Returns:
        tuple: input data X and labels y
    """
    # Create some additional features
    housing = data.copy()
    housing = data.drop(
        "median_house_value", axis=1
    )  # drop labels for training set
    housing_labels = data["median_house_value"].copy()

    # Impute the missing
    imputer = SimpleImputer(strategy="median")
    housing_num = housing.drop("ocean_proximity", axis=1)
    imputer.fit(housing_num)
    X = imputer.transform(housing_num)

    housing_tr = pd.DataFrame(
        X, columns=housing_num.columns, index=housing.index
    )
    housing_tr["rooms_per_household"] = (
        housing_tr["total_rooms"] / housing_tr["households"]
    )
    housing_tr["bedrooms_per_room"] = (
        housing_tr["total_bedrooms"] / housing_tr["total_rooms"]
    )
    housing_tr["population_per_household"] = (
        housing_tr["population"] / housing_tr["households"]
    )

    housing_cat = housing[["ocean_proximity"]]
    housing_prepared = housing_tr.join(
        pd.get_dummies(housing_cat, drop_first=True)
    )

    return housing_prepared, housing_labels



""" All configuration settings are available here"""

import os

DATA_DOWNLOAD_ROOT = "https://github.com/ageron/handson-ml/master/"
DEFAULT_HOUSING_PATH = "datasets"
HOUSING_URL = DATA_DOWNLOAD_ROOT + "datasets/housing/housing.tgz"


LOGS_PATH = "logs/logs.txt"



import mlflow

# Create an experiment name, which must be unique and case sensitive
experiment_id = mlflow.create_experiment("Random Forest")
experiment = mlflow.get_experiment(experiment_id)
print("Name: {}".format(experiment.name))
print("Experiment_id: {}".format(experiment.experiment_id))
print("Artifact Location: {}".format(experiment.artifact_location))
print("Tags: {}".format(experiment.tags))
print("Lifecycle_stage: {}".format(experiment.lifecycle_stage))



""" This module is for traing the data"""
import argparse
import os
import sys
import warnings
from urllib.parse import urlparse

import mlflow
import mlflow.sklearn
import pandas as pd
from sklearn.ensemble import RandomForestRegressor
from sklearn.linear_model import ElasticNet

from pavan_housing import logger
from pavan_housing.methods import *

warnings.filterwarnings("ignore")

# import the data
# import ingest_data

# define the logger
LOGGER = logger.get_logger(__name__)

# Parsing the command line inputs

DEFAULT_HOUSING_TRAIN_PATH = os.path.join("datasets", "train.csv")
DEFAULT_HOUSING_TEST_PATH = os.path.join("datasets", "test.csv")

parser = argparse.ArgumentParser()
parser.add_argument(
    "--train_path",
    help="Path to training data",
    default=DEFAULT_HOUSING_TRAIN_PATH,
)
parser.add_argument(
    "--test_path",
    help="Path to testing data",
    default=DEFAULT_HOUSING_TEST_PATH,
)

args = parser.parse_args()

HOUSING_TRAIN_PATH = args.train_path
HOUSING_TEST_PATH = args.test_path

# *********Model building and training

# Load the trainig data
train_data = pd.read_csv(HOUSING_TRAIN_PATH)
test_data = pd.read_csv(HOUSING_TEST_PATH)

# Load preprocessed X and y

train_x, train_y = preprocess(train_data)
test_x, test_y = preprocess(test_data)

with mlflow.start_run(experiment_id="1") as run:

    LOGGER.info("Elastic Net Training Started")
    #  Model1 -- Elastic Net
    alpha = float(sys.argv[1]) if len(sys.argv) > 1 else 0.5
    l1_ratio = float(sys.argv[2]) if len(sys.argv) > 2 else 0.5
    lr = ElasticNet(alpha=alpha, l1_ratio=l1_ratio, random_state=42)
    lr.fit(train_x, train_y)

    predicted_qualities = lr.predict(test_x)

    (rmse, mae, r2) = eval_metrics(test_y, predicted_qualities)

    print("Elasticnet model (alpha=%f, l1_ratio=%f):" % (alpha, l1_ratio))
    print("  RMSE: %s" % rmse)
    print("  MAE: %s" % mae)
    print("  R2: %s" % r2)

    mlflow.log_param("alpha", alpha)
    mlflow.log_param("l1_ratio", l1_ratio)
    mlflow.log_metric("rmse", rmse)
    mlflow.log_metric("r2", r2)
    mlflow.log_metric("mae", mae)

    tracking_url_type_store = urlparse(mlflow.get_tracking_uri()).scheme

    # Model registry does not work with file store
    if tracking_url_type_store != "file":
        mlflow.sklearn.log_model(
            lr, "model", registered_model_name="ElasticnetWineModel"
        )
    else:
        mlflow.sklearn.log_model(lr, "model")

with mlflow.start_run(experiment_id="2") as run:

    LOGGER.info("Random Forest Training Started")
    #  Model1 -- Elastic Net
    n_estimators = 100
    max_depth = 3
    rf = RandomForestRegressor(n_estimators=n_estimators, max_depth=max_depth)
    rf.fit(train_x, train_y)

    predicted_qualities = rf.predict(test_x)

    (rmse, mae, r2) = eval_metrics(test_y, predicted_qualities)

    print(
        "Random Forest model (n_estimators=%f, max_depth=%f):"
        % (n_estimators, max_depth)
    )
    print("  RMSE: %s" % rmse)
    print("  MAE: %s" % mae)
    print("  R2: %s" % r2)

    mlflow.log_param("alpha", n_estimators)
    mlflow.log_param("l1_ratio", max_depth)
    mlflow.log_metric("rmse", rmse)
    mlflow.log_metric("r2", r2)
    mlflow.log_metric("mae", mae)

    tracking_url_type_store = urlparse(mlflow.get_tracking_uri()).scheme

    # Model registry does not work with file store
    if tracking_url_type_store != "file":
        mlflow.sklearn.log_model(
            lr, "model", registered_model_name="RandomForestModel"
        )
    else:
        mlflow.sklearn.log_model(lr, "model")
